/* ---------------------------------------------------------------------------------------------------------------------------- */
/* CREATE THE DATABASE BEFORE EXECUTING THIS SCRIPT */
/* CREATE DB ROLES OpIRoleDBManagerFixed, OpIRoleDBClientFixed
	FOR SPECIFIED DATABASE*/
/* This procedure adds or replaces Roles, making them members of Fixed Roles to inherit their permissions 
	so that DB Users made Members of these Roles will inherit those permissions */
/* USE A SEPARATE DB ROLE SETUP SQL FILE FOR EACH OpI DATABASE */
/* ---------------------------------------------------------------------------------------------------------------------------- */

/* Security Architecture:  
	https://learn.microsoft.com/en-us/sql/relational-databases/security/authentication-access/getting-started-with-database-engine-permissions?view=sql-server-ver16
	https://www.mssqltips.com/sqlservertip/2699/sql-server-user-defined-server-roles/ */


/* Set up Role OpIDeveloper and OpIUser roles in specified Database to be included in OpIRoles */
USE SQLServerToolDB;	-- Specify the Database to which these DB Roles apply
GO
DECLARE	@roleName1 NVARCHAR(max) = NULL
DECLARE	@roleName2 NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@query NVARCHAR(max) = NULL
DECLARE	@schemaName NVARCHAR(max) = NULL
SET		@roleName1=N'OpIDeveloper'
SET		@roleName2=N'OpIUser'
SET		@activeNow = N'Yes'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No
SET		@schemaName=N'dbo'

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow = N'Yes'
	BEGIN
		/* Drop previously-set Database Fixed Role membership */
		SET @query = N'ALTER ROLE db_owner			'	+ N'DROP MEMBER '		+ @roleName1 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_securityadmin	'	+ N'DROP MEMBER '		+ @roleName1 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_accessadmin	'	+ N'DROP MEMBER '		+ @roleName1 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_backupoperator	'	+ N'DROP MEMBER '		+ @roleName1 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_ddladmin 		'	+ N'DROP MEMBER '		+ @roleName1 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_datawriter 	'	+ N'DROP MEMBER '		+ @roleName1 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_datareader 	'	+ N'DROP MEMBER '		+ @roleName1 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_denydatawriter '	+ N'DROP MEMBER '		+ @roleName1 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_denydatareader '	+ N'DROP MEMBER '		+ @roleName1 + N';';		EXEC sp_executesql @query;
	END

	SET @query=N'IF DATABASE_PRINCIPAL_ID(''' + @roleName1 + N''') IS NULL ' +
		N'CREATE ROLE [' + @roleName1 + '] AUTHORIZATION db_owner;'; EXEC sp_executesql @query;
	SET @query = N'GRANT CONTROL ON SCHEMA::'	+ @schemaName	+ N' TO [' + @roleName1 + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;


	IF	@replaceNow = N'Yes' 
	BEGIN 
		/* Drop previously-set Database Fixed Role membership */
		SET @query = N'ALTER ROLE db_owner			'	+ N'DROP MEMBER '		+ @roleName2 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_securityadmin	'	+ N'DROP MEMBER '		+ @roleName2 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_accessadmin	'	+ N'DROP MEMBER '		+ @roleName2 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_backupoperator	'	+ N'DROP MEMBER '		+ @roleName2 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_ddladmin 		'	+ N'DROP MEMBER '		+ @roleName2 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_datawriter 	'	+ N'DROP MEMBER '		+ @roleName2 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_datareader 	'	+ N'DROP MEMBER '		+ @roleName2 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_denydatawriter '	+ N'DROP MEMBER '		+ @roleName2 + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_denydatareader '	+ N'DROP MEMBER '		+ @roleName2 + N';';		EXEC sp_executesql @query;
	END

	SET @query=N'IF DATABASE_PRINCIPAL_ID(''' + @roleName2 + N''') IS NULL ' +
		N'CREATE ROLE [' + @roleName2 + '] AUTHORIZATION db_owner;'; EXEC sp_executesql @query; 
	SET @query = N'GRANT EXECUTE ON SCHEMA::'	+ @schemaName	+ N' TO [' + @roleName2 + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
END
GO



/* SET up Role OpIRoleDBManagerFixed for specified Database */
USE SQLServerToolDB; 	-- Specify the Database to which these DB Roles apply
GO
DECLARE	@roleName NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@query NVARCHAR(max) = NULL
DECLARE	@schemaName NVARCHAR(max) = NULL
SET		@roleName=N'OpIRoleDBManagerFixed'
SET		@activeNow = N'No'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No
SET		@schemaName=N'dbo'

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow=N'Yes'
	BEGIN
		/* Drop previously-set Database Fixed Role membership */
		SET @query = N'ALTER ROLE db_owner			'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_securityadmin	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_accessadmin	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_backupoperator	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_ddladmin 		'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_datawriter 	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_datareader 	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_denydatawriter '	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_denydatareader '	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		/* User Roles OpIDeveloper and OpIUser may or may not exist; Remove this MEMBER if the Role exists. */
		SET @query=N'IF NOT DATABASE_PRINCIPAL_ID(''OpIDeveloper'') IS NULL ' +
			N'ALTER ROLE OpIDeveloper 				'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query=N'IF NOT DATABASE_PRINCIPAL_ID(''OpIUser'') IS NULL ' +
			N'ALTER ROLE OpIUser 					'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
	END
	
	/* Create or Re-Create the Role */
	SET @query=N'IF DATABASE_PRINCIPAL_ID(''' + @roleName + N''') IS NULL ' +
		 N'CREATE ROLE [' + @roleName + '] AUTHORIZATION db_owner;'; EXEC sp_executesql @query;

	/* Set Database Fixed Role membership of this Role */
	SET @query = N'ALTER ROLE db_securityadmin '		+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER ROLE db_accessadmin '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER ROLE db_ddladmin '				+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER ROLE db_datawriter '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER ROLE db_datareader '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER ROLE OpIDeveloper '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
END
GO



/* SET up Role OpIRoleDBClientFixed for specified Database */
USE SQLServerToolDB; 	-- Specify the Database to which these DB Roles apply
GO
DECLARE	@roleName NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@query NVARCHAR(max) = NULL
DECLARE	@schemaName NVARCHAR(max) = NULL
SET		@roleName=N'OpIRoleDBClientFixed'
SET		@activeNow = N'No'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No
SET		@schemaName=N'dbo'

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow=N'Yes'
	BEGIN
		/* Drop previously-set Database Fixed Role membership */
		SET @query = N'ALTER ROLE db_owner			'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_securityadmin	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_accessadmin	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_backupoperator	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_ddladmin 		'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_datawriter 	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_datareader 	'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_denydatawriter '	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query = N'ALTER ROLE db_denydatareader '	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		/* User Roles OpIDeveloper and OpIUser may or may not exist; Remove this MEMBER if the Role exists. */
		SET @query=N'IF NOT DATABASE_PRINCIPAL_ID(''OpIDeveloper'') IS NULL ' +
			N'ALTER ROLE OpIDeveloper 				'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
		SET @query=N'IF NOT DATABASE_PRINCIPAL_ID(''OpIUser'') IS NULL ' +
			N'ALTER ROLE OpIUser 					'	+ N'DROP MEMBER '		+ @roleName + N';';		EXEC sp_executesql @query;
	END

	/* Create or Re-Create the Role */
	SET @query=N'IF DATABASE_PRINCIPAL_ID(''' + @roleName + N''') IS NULL ' +
		 N'CREATE ROLE [' + @roleName + '] AUTHORIZATION db_owner;'; EXEC sp_executesql @query;

	/* Set Database Fixed Role membership of this Role */
	SET @query = N'ALTER ROLE db_accessadmin '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER ROLE db_datareader '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER ROLE OpIUser '					+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
END
GO