/* ---------------------------------------------------------------------------------------------------------------------------- */
/* CREATE DATABASE USERS FOR SPECIFIED DATABASE. */
/* CREATE THE DATABASE BEFORE EXECUTING THIS SCRIPT */
/* USE A SEPARATE DB USER SETUP SQL FILE FOR EACH OpI DATABASE */
/* REPEAT THE SECTION BELOW FOR EACH DATABASE AND EACH DATABASE LOGIN */
/* ---------------------------------------------------------------------------------------------------------------------------- */

/* Security Architecture:  
	https://learn.microsoft.com/en-us/sql/relational-databases/security/authentication-access/getting-started-with-database-engine-permissions?view=sql-server-ver16
	https://www.mssqltips.com/sqlservertip/2699/sql-server-user-defined-server-roles/ */


/* CREATE USER OpIManager FOR SPECIFIED DATABASE */
USE SQLServerToolDB;	-- For each User setup, specify the Database for the User
GO
DECLARE @OpI_LoginName NVARCHAR(30) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE @roleName NVARCHAR(max) = NULL
DECLARE @query NVARCHAR(max) = NULL
SET		@OpI_LoginName=N'OpIManager';				-- Specify a Server Login so the Database User name will match the Login
SET		@activeNow = N'Yes'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No
SET		@roleName = N'OpIRoleDBManager'		-- OpIRoleDBSuper, OpIRoleDBManager, OpIRoleDBDataOwner, OpIRoleDBClientLocal, OpIRoleDBClientRemote

IF @activeNow = N'Yes'
BEGIN
	IF @replaceNow = N'Yes' BEGIN SET @query = N'DROP USER IF EXISTS ' + @OpI_LoginName + N';';	EXEC sp_executesql @query; END
	IF EXISTS (SELECT [sys].[sql_logins].[name], [sys].[sql_logins].[type] 
		FROM [sys].[sql_logins] 
		WHERE [type] IN (N'U', N'S') AND [name] = @OpI_LoginName) 
	BEGIN
		IF NOT EXISTS (SELECT [sys].[database_principals].[name], [sys].[database_principals].[type] 
			FROM [sys].[database_principals] 
			WHERE [type] IN (N'U', N'S') AND [name] = @OpI_LoginName) 
			BEGIN 
				SET @query = N'CREATE USER  [' + @OpI_LoginName + '] FROM LOGIN [' + @OpI_LoginName + N'];'; EXEC sp_executesql @query;
			END;
		SET @query = N'IF NOT DATABASE_PRINCIPAL_ID(''' + @roleName + N''') IS NULL ' + 
			N'ALTER ROLE ' + @roleName + N' ADD MEMBER ' + @OpI_LoginName + N';'; EXEC sp_executesql @query;

		/* Add membership to OpI Fixed-Role Membership Role if it exists */
		SET @query = N'IF NOT DATABASE_PRINCIPAL_ID(''' + @roleName + N'Fixed' + N''') IS NULL ' + 
			N'ALTER ROLE ' + @roleName + N'Fixed' + N' ADD MEMBER ' + @OpI_LoginName + ';'; EXEC sp_executesql @query;
	END;
END
GO



/* CREATE USER OpIClient FOR SPECIFIED DATABASE */
USE SQLServerToolDB;	-- For each User setup, specify the Database for the User
GO
DECLARE @OpI_LoginName NVARCHAR(30) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE @roleName NVARCHAR(max) = NULL
DECLARE @query NVARCHAR(max) = NULL
SET		@OpI_LoginName=N'OpIClient';				-- Specify a Server Login so the Database User name will match the Login
SET		@activeNow = N'Yes'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No
SET		@roleName = N'OpIRoleDBClient'		-- OpIRoleDBSuper, OpIRoleDBManager, OpIRoleDBDataOwner, OpIRoleDBClient

IF @activeNow = N'Yes'
BEGIN
	IF @replaceNow = N'Yes' BEGIN SET @query = N'DROP USER IF EXISTS ' + @OpI_LoginName + N';';	EXEC sp_executesql @query; END
	IF EXISTS (SELECT [sys].[sql_logins].[name], [sys].[sql_logins].[type] 
		FROM [sys].[sql_logins] 
		WHERE [type] IN (N'U', N'S') AND [name] = @OpI_LoginName) 
	BEGIN
		IF NOT EXISTS (SELECT [sys].[database_principals].[name], [sys].[database_principals].[type] 
			FROM [sys].[database_principals] 
			WHERE [type] IN (N'U', N'S') AND [name] = @OpI_LoginName) 
			BEGIN 
				SET @query = N'CREATE USER  [' + @OpI_LoginName + '] FROM LOGIN [' + @OpI_LoginName + N'];'; EXEC sp_executesql @query;
			END;
		SET @query = N'IF NOT DATABASE_PRINCIPAL_ID(''' + @roleName + N''') IS NULL ' + 
			N'ALTER ROLE ' + @roleName + N' ADD MEMBER ' + @OpI_LoginName + N';'; EXEC sp_executesql @query;
		
		/* Add membership to OpI Fixed-Role Membership Role if it exists */
		SET @query = N'IF NOT DATABASE_PRINCIPAL_ID(''' + @roleName + N'Fixed' + N''') IS NULL ' + 
			N'ALTER ROLE ' + @roleName + N'Fixed' + N' ADD MEMBER ' + @OpI_LoginName + ';'; EXEC sp_executesql @query;
	END;
END
GO



/* CREATE USER ProgramManagement FOR SPECIFIED DATABASE */
USE SQLServerToolDB;	-- For each User setup, specify the Database for the User
GO
DECLARE @OpI_LoginName NVARCHAR(30) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE @roleName NVARCHAR(max) = NULL
DECLARE @query NVARCHAR(max) = NULL
SET		@OpI_LoginName=N'ProgramManagement';				-- Specify a Server Login so the Database User name will match the Login
SET		@activeNow = N'Yes'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No
SET		@roleName = N'OpIRoleDBManager'		-- OpIRoleDBSuper, OpIRoleDBManager, OpIRoleDBDataOwner, OpIRoleDBClientLocal, OpIRoleDBClientRemote

IF @activeNow = N'Yes'
BEGIN
	IF @replaceNow = N'Yes' BEGIN SET @query = N'DROP USER IF EXISTS ' + @OpI_LoginName + N';';	EXEC sp_executesql @query; END
	IF EXISTS (SELECT [sys].[sql_logins].[name], [sys].[sql_logins].[type] 
		FROM [sys].[sql_logins] 
		WHERE [type] IN (N'U', N'S') AND [name] = @OpI_LoginName) 
	BEGIN
		IF NOT EXISTS (SELECT [sys].[database_principals].[name], [sys].[database_principals].[type] 
			FROM [sys].[database_principals] 
			WHERE [type] IN (N'U', N'S') AND [name] = @OpI_LoginName) 
			BEGIN 
				SET @query = N'CREATE USER  [' + @OpI_LoginName + '] FROM LOGIN [' + @OpI_LoginName + N'];'; EXEC sp_executesql @query;
			END;
		SET @query = N'IF NOT DATABASE_PRINCIPAL_ID(''' + @roleName + N''') IS NULL ' + 
			N'ALTER ROLE ' + @roleName + N' ADD MEMBER ' + @OpI_LoginName + N';'; EXEC sp_executesql @query;

		/* Add membership to OpI Fixed-Role Membership Role if it exists */
		SET @query = N'IF NOT DATABASE_PRINCIPAL_ID(''' + @roleName + N'Fixed' + N''') IS NULL ' + 
			N'ALTER ROLE ' + @roleName + N'Fixed' + N' ADD MEMBER ' + @OpI_LoginName + ';'; EXEC sp_executesql @query;
	END;
END
GO


/* CREATE USER ProgramManagement2 FOR SPECIFIED DATABASE */
USE SQLServerToolDB;	-- For each User setup, specify the Database for the User
GO
DECLARE @OpI_LoginName NVARCHAR(30) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE @roleName NVARCHAR(max) = NULL
DECLARE @query NVARCHAR(max) = NULL
SET		@OpI_LoginName=N'ProgramManagement2';				-- Specify a Server Login so the Database User name will match the Login
SET		@activeNow = N'Yes'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No
SET		@roleName = N'OpIRoleDBManager'		-- OpIRoleDBSuper, OpIRoleDBManager, OpIRoleDBDataOwner, OpIRoleDBClientLocal, OpIRoleDBClientRemote

IF @activeNow = N'Yes'
BEGIN
	IF @replaceNow = N'Yes' BEGIN SET @query = N'DROP USER IF EXISTS ' + @OpI_LoginName + N';';	EXEC sp_executesql @query; END
	IF EXISTS (SELECT [sys].[sql_logins].[name], [sys].[sql_logins].[type] 
		FROM [sys].[sql_logins] 
		WHERE [type] IN (N'U', N'S') AND [name] = @OpI_LoginName) 
	BEGIN
		IF NOT EXISTS (SELECT [sys].[database_principals].[name], [sys].[database_principals].[type] 
			FROM [sys].[database_principals] 
			WHERE [type] IN (N'U', N'S') AND [name] = @OpI_LoginName) 
			BEGIN 
				SET @query = N'CREATE USER  [' + @OpI_LoginName + '] FROM LOGIN [' + @OpI_LoginName + N'];'; EXEC sp_executesql @query;
			END;
		SET @query = N'IF NOT DATABASE_PRINCIPAL_ID(''' + @roleName + N''') IS NULL ' + 
			N'ALTER ROLE ' + @roleName + N' ADD MEMBER ' + @OpI_LoginName + N';'; EXEC sp_executesql @query;

		/* Add membership to OpI Fixed-Role Membership Role if it exists */
		SET @query = N'IF NOT DATABASE_PRINCIPAL_ID(''' + @roleName + N'Fixed' + N''') IS NULL ' + 
			N'ALTER ROLE ' + @roleName + N'Fixed' + N' ADD MEMBER ' + @OpI_LoginName + ';'; EXEC sp_executesql @query;
	END;
END
GO