/* ---------------------------------------------------------------------------------------------------------------------------- */
/* CREATE SVR ROLES OpIRoleSvrSuper, OpIRoleSvrManager, OpIRoleSvrDataOwner, OpIRoleSvrClient
	FOR SERVER INSTANCE <SERVERNAME>\<INSTANCENAME> FOR ASSIGNMENT OF PERMISSIONS TO EACH SERVER LOGIN */
/* ---------------------------------------------------------------------------------------------------------------------------- */

/* Security Architecture:  
	https://learn.microsoft.com/en-us/sql/relational-databases/security/authentication-access/getting-started-with-database-engine-permissions?view=sql-server-ver16
	https://www.mssqltips.com/sqlservertip/2699/sql-server-user-defined-server-roles/ 
	https://learn.microsoft.com/en-us/sql/relational-databases/system-catalog-views/sys-server-role-members-transact-sql?view=sql-server-ver16 */

USE master;
GO
/* Grant EXECUTE to OLEDB_Provider */
GRANT EXECUTE ON SYS.XP_PROP_OLEDB_PROVIDER TO [Public];
GO

USE master;	-- For Server Level Roles, always master DB
GO

/* For Server, create Roles OpIRoleSvrSuper, OpIRoleSvrManager, OpIRoleSvrDataOwner - No Svr Capabilities for OpIRoleSvrClient */
/* SET UP ONCE ONLY FOR THE SERVER INSTANCE <INSTANCENAME> */

/* SET UP OpIRoleSvrSuper */
DECLARE	@roleName NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@query NVARCHAR(max) = NULL
SET		@roleName=N'OpIRoleSvrSuper'
SET		@activeNow = N'Yes'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow = N'Yes'
	BEGIN
		/* DROP previous membership inServer Fixed Roles */
		SET @query = N'ALTER SERVER ROLE serveradmin	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE securityadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE processadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE setupadmin 	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE bulkadmin		'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE diskadmin 		'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE dbcreator 		'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		/* Then Drop previous instance for this role */
		SET @query = N'IF EXISTS(SELECT [sys].[server_principals].[name], [sys].[server_principals].[type] ' +
		N'	FROM [sys].[server_principals] ' +
        N'	WHERE [type] IN (N''R'') AND [name]= ''' + @roleName + N''')' +
		N'	DROP SERVER ROLE [' + @roleName + N'];';	EXEC sp_executesql @query;
	END

	SET @query = N'IF NOT EXISTS (SELECT [sys].[server_principals].[name], [sys].[server_principals].[type] ' +
    N'	FROM [sys].[server_principals] ' +
    N'	WHERE [type] IN (N''R'') AND [name]= ''' + @roleName + N''')' +
	N'	CREATE SERVER ROLE [' + @roleName + '] AUTHORIZATION sysadmin;';	EXEC sp_executesql @query;
	
	/* Configure server-level permissions to the role using GRANT, DENY, REVOKE:  https://learn.microsoft.com/en-us/sql/t-sql/statements/create-server-role-transact-sql?view=sql-server-ver16 */
	/* Server Fixed Role sysadmin */
	SET @query = N'GRANT CONTROL SERVER					'	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Server Fixed Role serveradmin */
	SET @query = N'GRANT ALTER SETTINGS					'	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Server Fixed Role securityadmin */
	SET @query = N'GRANT ALTER ANY LOGIN '					+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Server Fixed Role setupadmin */
	SET @query = N'GRANT ALTER ANY LINKED SERVER		'	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Server Fixed Role dbcreator */
	SET @query = N'GRANT ALTER ANY DATABASE				'	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	SET @query = N'GRANT CREATE ANY DATABASE			'	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
END
GO



/* SET UP OpIRoleSvrManager */
DECLARE	@roleName NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@query NVARCHAR(max) = NULL
SET		@roleName=N'OpIRoleSvrManager'
SET		@activeNow = N'Yes'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow = N'Yes'
	BEGIN
		/* DROP previous membership inServer Fixed Roles */
		SET @query = N'ALTER SERVER ROLE serveradmin	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE securityadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE processadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE setupadmin 	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE bulkadmin		'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE diskadmin 		'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE dbcreator 		'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		/* Then Drop previous instance for this role */
		SET @query = N'IF EXISTS(SELECT [sys].[server_principals].[name], [sys].[server_principals].[type] ' +
		N'	FROM [sys].[server_principals] ' +
        N'	WHERE [type] IN (N''R'') AND [name]= ''' + @roleName + N''')' +
		N'	DROP SERVER ROLE [' + @roleName + N'];';	EXEC sp_executesql @query;
	END

	SET @query = N'IF NOT EXISTS (SELECT [sys].[server_principals].[name], [sys].[server_principals].[type] ' +
    N'	FROM [sys].[server_principals] ' +
    N'	WHERE [type] IN (N''R'') AND [name]= ''' + @roleName + N''')' +
	N'	CREATE SERVER ROLE [' + @roleName + '] AUTHORIZATION sysadmin;';	EXEC sp_executesql @query;
	
	/* Configure server-level permissions to the role using GRANT, DENY, REVOKE:  https://learn.microsoft.com/en-us/sql/t-sql/statements/create-server-role-transact-sql?view=sql-server-ver16 */
	/* Server Fixed Role serveradmin */
	SET @query = N'GRANT ALTER SETTINGS					'	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Server Fixed Role securityadmin */
	SET @query = N'GRANT ALTER ANY LOGIN				'	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Server Fixed Role setupadmin */
	SET @query = N'GRANT ALTER ANY LINKED SERVER		'	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Server Fixed Role dbcreator */
	SET @query = N'GRANT ALTER ANY DATABASE				'	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	SET @query = N'GRANT CREATE ANY DATABASE			'	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
END
GO



/* SET UP OpIRoleSvrDataOwner */
DECLARE	@roleName NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@query NVARCHAR(max) = NULL
SET		@roleName=N'OpIRoleSvrDataOwner'
SET		@activeNow = N'Yes'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow = N'Yes'
	BEGIN
		/* DROP previous membership inServer Fixed Roles */
		SET @query = N'ALTER SERVER ROLE serveradmin	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE securityadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE processadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE setupadmin 	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE bulkadmin		'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE diskadmin 		'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE dbcreator 		'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		/* Then Drop previous instance for this role */
		SET @query = N'IF EXISTS(SELECT [sys].[server_principals].[name], [sys].[server_principals].[type] ' +
		N'	FROM [sys].[server_principals] ' +
        N'	WHERE [type] IN (N''R'') AND [name]= ''' + @roleName + N''')' +
		N'	DROP SERVER ROLE [' + @roleName + N'];';	EXEC sp_executesql @query;
	END

	SET @query = N'IF NOT EXISTS (SELECT [sys].[server_principals].[name], [sys].[server_principals].[type] ' +
    N'	FROM [sys].[server_principals] ' +
    N'	WHERE [type] IN (N''R'') AND [name]= ''' + @roleName + N''')' +
	N'	CREATE SERVER ROLE [' + @roleName + '] AUTHORIZATION sysadmin;';	EXEC sp_executesql @query;
	
	/* Configure server-level permissions to the role using GRANT, DENY, REVOKE:  https://learn.microsoft.com/en-us/sql/t-sql/statements/create-server-role-transact-sql?view=sql-server-ver16 */
	/* Server Fixed Role serveradmin */
	SET @query = N'GRANT ALTER SETTINGS					'	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Server Fixed Role setupadmin */
	SET @query = N'GRANT ALTER ANY LINKED SERVER		'	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
	/* Server Fixed Role dbcreator */
	SET @query = N'GRANT ALTER ANY DATABASE				 '	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
END
GO



/* SET UP OpIRoleSvrClient */
DECLARE	@roleName NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@query NVARCHAR(max) = NULL
SET		@roleName=N'OpIRoleSvrClient'
SET		@activeNow = N'Yes'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow = N'Yes'
	BEGIN
		/* DROP previous membership inServer Fixed Roles */
		SET @query = N'ALTER SERVER ROLE serveradmin	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE securityadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE processadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE setupadmin 	'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE bulkadmin		'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE diskadmin 		'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE dbcreator 		'	+ N'DROP MEMBER '	+ @roleName + N';';						EXEC sp_executesql @query;
		/* Then Drop previous instance for this role */
		SET @query = N'IF EXISTS(SELECT [sys].[server_principals].[name], [sys].[server_principals].[type] ' +
		N'	FROM [sys].[server_principals] ' +
        N'	WHERE [type] IN (N''R'') AND [name]= ''' + @roleName + N''')' +
		N'	DROP SERVER ROLE [' + @roleName + N'];';	EXEC sp_executesql @query;
	END

	SET @query = N'IF NOT EXISTS (SELECT [sys].[server_principals].[name], [sys].[server_principals].[type] ' +
    N'	FROM [sys].[server_principals] ' +
    N'	WHERE [type] IN (N''R'') AND [name]= ''' + @roleName + N''')' +
	N'	CREATE SERVER ROLE [' + @roleName + '] AUTHORIZATION sysadmin;';	EXEC sp_executesql @query;
	
	/* Configure server-level permissions to the role using GRANT, DENY, REVOKE:  https://learn.microsoft.com/en-us/sql/t-sql/statements/create-server-role-transact-sql?view=sql-server-ver16 */
	/* Server Fixed Role serveradmin */
	-- No Server Permissions for OpISvrRoleClient
	-- SET @query = N'GRANT ALTER SETTINGS					 '	+ N' TO ['			+ @roleName + N'] WITH GRANT OPTION;';	EXEC sp_executesql @query;
END
GO